Module BinaryCompare

    Dim Binary1() As Byte
    Dim Binary2() As Byte
    Public TestFiles_ListDetails As Boolean = True


    Public Sub CompareBinaries(Optional ByVal SortByPosition As Boolean = False)

        Dim File1 As String = ""
        Dim File2 As String = ""
        Dim CalibrationFile1 As String = ""
        Dim CalibrationFile2 As String = ""

        Dim MissingBinary As Boolean = True
        'Get the location to the first binary file
        Dim Open_Dialog As New OpenFileDialog
        Open_Dialog.Filter = "Binary Files (*.bin)|*.bin|All Files (*.*)|*.*"
        'Open_Dialog.InitialDirectory = Default_Binary_Path
        Open_Dialog.ShowDialog()
        If Not Open_Dialog.FileName = "" Then
            File1 = Open_Dialog.FileName
            'Get the location to the second binary file
            Open_Dialog.Filter = "Binary Files (*.bin)|*.bin|All Files (*.*)|*.*"
            'Open_Dialog.InitialDirectory = Default_Binary_Path
            Open_Dialog.ShowDialog()
            If Not Open_Dialog.FileName = "" Then
                File2 = Open_Dialog.FileName
                MissingBinary = False
            End If
        End If

        Dim MissingCalFile As Boolean = True

        If Not MissingBinary Then
            'Verify we have access to the calibration files for each binary
            Dim tmpOSID1 As String = GetMapNumber(Get_OSID_Compare(File1))  'Get_OSID_Compare(File1)
            Dim tmpOSID2 As String = GetMapNumber(Get_OSID_Compare(File2))  'Get_OSID_Compare(File2)

            If tmpOSID1 = tmpOSID2 Then
                If Not tmpOSID1 = "" And Not tmpOSID2 = "" Then
                    CalibrationFile1 = tmpOSID1  'FindCalFile(tmpOSID1)
                    CalibrationFile2 = tmpOSID2  'FindCalFile(tmpOSID2)
                    If Not CalibrationFile1 = "" And Not CalibrationFile2 = "" Then
                        MissingCalFile = False
                    End If
                End If


                Dim tmpSaveFile As String = ""
                If Not MissingCalFile And Not MissingBinary Then
                    'We have everything we need, start comparing the files...
                    Dim Save_Dialog As New SaveFileDialog
                    Save_Dialog.Filter = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*"
                    Save_Dialog.ShowDialog()
                    If Save_Dialog.FileName <> "" Then
                        tmpSaveFile = Save_Dialog.FileName

                        TestFiles(File1, CalibrationFile1, File2, CalibrationFile2, tmpSaveFile, SortByPosition)

                        If MessageBox.Show("Finished Comparing Binaries." & vbCrLf & vbCrLf & "Would you like to view the report?", "View Report", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
                            Shell("notepad.exe " & tmpSaveFile, AppWinStyle.MaximizedFocus)
                        End If

                    End If
                End If
            Else
                MessageBox.Show("The 2 binary files do not use the same map - can not compare at this time...")
                MissingCalFile = False  'set to false to skip the error message after leaving If-Then 
            End If

        End If


        If MissingCalFile Then

            If CalibrationFile1 = "" Then CalibrationFile1 = "Missing"
            If CalibrationFile2 = "" Then CalibrationFile2 = "Missing"

            MessageBox.Show("Could not find the calibration file for at least one of the binary files." & vbCrLf & vbCrLf & _
                            "Without the calibration file, there is no way to determin where each table" & vbCrLf & _
                            "is for each of the two binaries." & vbCrLf & vbCrLf & _
                            "Calibration File 1: " & CalibrationFile1 & vbCrLf & _
                            "Calibration File 2: " & CalibrationFile2, "Missing Calibration File", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End If


    End Sub
    Private Sub TestFiles(ByVal bin1 As String, ByVal cal1 As String, ByVal bin2 As String, ByVal cal2 As String, ByVal tmpSaveFile As String, Optional ByVal SoryByPosition As Boolean = False)

        If Not IO.File.Exists(DatabasePath) Then Exit Sub

        'Setup the dataset - 2 tables, one for each binary file
        Dim cn As New OleDb.OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & DatabasePath)
        cn.Open()

        Dim dsDiff As New DataSet
        Dim strSQL As String = "select * from TableData where MapNumber=" & Val(cal1) & " order by TableName"
        Dim da As New OleDb.OleDbDataAdapter(strSQL, cn)

        da.Fill(dsDiff, "bin1")
        strSQL = "select * from TableData where MapNumber=" & Val(cal2) & " order by TableName"

        da.SelectCommand.CommandText = strSQL
        da.Fill(dsDiff, "bin2")

        cn.Close()



        'Start the report writer and setup variables

        Dim Difference_Count As Int16 = 0
        Dim tmpWriter As New IO.StreamWriter(tmpSaveFile, False)

        tmpWriter.WriteLine(" ")
        tmpWriter.WriteLine("This report was created on " & Date.Today.ToLongDateString & " at " & Now.ToLongTimeString)
        tmpWriter.WriteLine(" ")
        tmpWriter.WriteLine("The original report was saved as " & tmpSaveFile)
        tmpWriter.WriteLine(" ")

        Dim tablecount1 As Int16 = 0
        Dim tablecount2 As Int16 = 0
        Dim LowerTableCount As Integer = 0

        tmpWriter.WriteLine("Binary File #1: " & bin1)
        LoadBinaryFile(bin1, 1)
        tmpWriter.WriteLine("Binary File #2: " & bin2)
        LoadBinaryFile(bin2, 2)

        tablecount1 = dsDiff.Tables(0).Rows.Count  'Val(readINI.ReadINI.sGetINI(cal1, "Main", "Table_Count", 0))
        tablecount2 = dsDiff.Tables(1).Rows.Count  'Val(readINI.ReadINI.sGetINI(cal2, "Main", "Table_Count", 0))

        tmpWriter.WriteLine("")
        tmpWriter.WriteLine("Number of tables in File #1: " & tablecount1.ToString)
        tmpWriter.WriteLine("Number of tables in File #2: " & tablecount2.ToString)


        'Setup the progress bar
        Dim tmpPGBar As New ProgressDialog
        tmpPGBar.Height = 86
        tmpPGBar.Text = "Checking Tables..."
        tmpPGBar.StartPosition = FormStartPosition.CenterScreen  ' FormStartPosition.CenterParent
        tmpPGBar.Show()
        tmpPGBar.ProgressBar1.Minimum = 0
        tmpPGBar.ProgressBar1.Maximum = tablecount1 + 1

        'Figure which MAP has the greater number of tables
        Dim Table_Count As Int16
        If tablecount1 < tablecount2 Then
            Table_Count = tablecount1
            LowerTableCount = 1
        Else
            Table_Count = tablecount2
            LowerTableCount = 2
        End If


        If Table_Count > 0 Then

            Dim tmpString As String = ""
            Dim i As Integer = 0
            Dim a As Integer = 0
            Dim b As Long = 0
            Dim Table1 As String = ""
            Dim Table2 As String = ""

            Dim t1 As Long = 0
            Dim t2 As Long = 0
            Dim t3 As Long = 0

            Dim strTable1Size As String = ""
            Dim strTable2Size As String = ""

            Dim table1size As Int16 = 0
            Dim table2size As Int16 = 0

            Dim Table1Start As Long = 0
            Dim Table2Start As Long = 0
            Dim TablesAreDifferent As Boolean = False

            Dim tmpByte1 As Byte = 0
            Dim tmpByte2 As Byte = 0

            Dim Hex1 As String = ""
            Dim Hex2 As String = ""

            tmpWriter.WriteLine(" ")

            tmpPGBar.ProgressBar1.Value = 0

            TestFiles_ListDetails = False
            If MessageBox.Show("Do you want to list details about the differences?" & vbCrLf & vbCrLf & _
                               "Choose No if you only want a list of table that are different.", "", MessageBoxButtons.YesNo) = DialogResult.Yes Then
                TestFiles_ListDetails = True
            End If

            'Start the compare loop
            Dim z As Integer
            For z = 0 To tablecount1 - 1

                'If SoryByPosition Then
                ' i = SortArray(z)
                ' Else
                i = z
                'End If


                Table1 = dsDiff.Tables(0).Rows(i).Item("TableName")  'readINI.ReadINI.sGetINI(cal1, i.ToString, "Table_Name", "Table1")
                Table2 = dsDiff.Tables(1).Rows(i).Item("TableName")

                Application.DoEvents()
                tmpPGBar.Label1.Text = Table1
                tmpPGBar.ProgressBar1.Value = z
                tmpPGBar.Refresh()

                'If Table1.ToLower = "volumetric efficiency" Then
                'MsgBox("Volumetric Efficiency")
                'End If

                'No need to do the following since we are looking at the same MAP number
                'If Table1 = readINI.ReadINI.sGetINI(cal2, i.ToString, "Table_Name", "Table2") Then
                ' a = i
                ' table2 = readINI.ReadINI.sGetINI(cal2, i.ToString, "Table_Name", "Table2")
                ' Else
                ' For a = 1 To tablecount2
                ' table2 = readINI.ReadINI.sGetINI(cal2, a.ToString, "Table_Name", "Table2")
                ' If table2 = Table1 Then Exit For
                'Next
                'End If

                TablesAreDifferent = False
                If Table2 = Table1 Then
                    t1 = Val(dsDiff.Tables(0).Rows(i).Item("RowCount"))
                    t2 = Val(dsDiff.Tables(0).Rows(i).Item("ColumnCount"))
                    t3 = Val(dsDiff.Tables(0).Rows(i).Item("ElementSize"))
                    strTable1Size = t1.ToString & " x " & t2.ToString
                    table1size = (t1 * t2) * t3

                    t1 = Val(dsDiff.Tables(1).Rows(i).Item("RowCount"))
                    t2 = Val(dsDiff.Tables(1).Rows(i).Item("ColumnCount"))
                    t3 = Val(dsDiff.Tables(1).Rows(i).Item("ElementSize"))
                    strTable2Size = t1.ToString & " x " & t2.ToString
                    table2size = (t1 * t2) * t3

                    If table1size = table2size Then
                        Table1Start = Convert.ToInt32(dsDiff.Tables(0).Rows(i).Item("StartPosition"), 16)
                        Table2Start = Convert.ToInt32(dsDiff.Tables(1).Rows(i).Item("StartPosition"), 16)
                        For b = 0 To table1size - 1
                            tmpByte1 = Binary1(b + Table1Start)
                            tmpByte2 = Binary2(b + Table2Start)
                            If tmpByte1 <> tmpByte2 Then
                                TablesAreDifferent = True
                                Table1 = dsDiff.Tables(0).Rows(i).Item("TableName")
                                If Table1.ToUpper = "PROM_ID" Then
                                    Table1 = "OSID"
                                End If

                                Difference_Count += 1

                                If TestFiles_ListDetails Then
                                    tmpWriter.WriteLine("Table Name: " & Table1)
                                    'tmpWriter.WriteLine("     Table 1 Size: " & table1size.ToString & " - " & strTable1Size & "     Table 2 Size: " & table2size.ToString & " - " & strTable2Size)
                                    tmpWriter.WriteLine("     Table 1 Size: " & table1size.ToString & "     Table 2 Size: " & table2size.ToString)
                                    tmpWriter.WriteLine("     Table 1 Start Position: " & Hex(Table1Start) & "     Table 2 Start Position: " & Hex(Table2Start))

                                    Hex1 = Hex(tmpByte1)
                                    If Hex1.Length < 2 Then Hex1 = "0" & Hex1
                                    Hex2 = Hex(tmpByte2)
                                    If Hex2.Length < 2 Then Hex2 = "0" & Hex2

                                    tmpWriter.WriteLine("     Table 1 First Different Value Location: " & Hex(b + Table1Start) & "     Table 1 Value: " & tmpByte1 & " (" & Hex1 & ")")
                                    tmpWriter.WriteLine("     Table 2 First Different Value Location: " & Hex(b + Table2Start) & "     Table 2 Value: " & tmpByte2 & " (" & Hex2 & ")")
                                    tmpWriter.WriteLine(" ")
                                Else
                                    'Write nothing but the table name (Table1) to the output file, this is going to 
                                    'be a simple list of the tables that are different with no details.
                                    tmpWriter.WriteLine(Table1)
                                End If

                                Exit For
                            End If
                        Next

                    Else
                        tmpWriter.WriteLine("Table Name: " & Table1)
                        tmpWriter.WriteLine("     Table 1 Size: " & table1size.ToString & "     Table 2 Size: " & table2size.ToString)
                        tmpWriter.WriteLine("     Table 1 Start Position: " & Table1Start.ToString & "     Table 2 Start Position: " & Table2Start)
                        tmpWriter.WriteLine("     Tables are different sizes and can not be compared.")
                        tmpWriter.WriteLine(" ")
                    End If

                    'Exit For
                End If

            Next


            tmpWriter.WriteLine(" ")
            tmpWriter.WriteLine("There were " & Difference_Count.ToString & " tables that were different.")


        End If

        tmpWriter.Close()
        tmpPGBar.Close()
        da = Nothing
        dsDiff = Nothing

    End Sub
    Private Sub LoadBinaryFile(ByVal FileName As String, ByVal BinaryArray As Int16)

        Dim i As Integer
        Dim Imported_File_Size As Long

        Dim File_Info As System.IO.FileInfo = My.Computer.FileSystem.GetFileInfo(FileName)
        Imported_File_Size = File_Info.Length

        ' Create the reader for data.
        Dim fs As New IO.FileStream(FileName, IO.FileMode.Open, IO.FileAccess.Read)
        Dim r As New IO.BinaryReader(fs)
        Dim tmpByte As Byte
        Dim e As New System.Windows.Forms.DataGridViewRow

        If BinaryArray = 1 Then
            ReDim Binary1(Imported_File_Size - 1)
        Else
            ReDim Binary2(Imported_File_Size - 1)
        End If

        For i = 0 To Imported_File_Size - 1

            tmpByte = r.ReadByte

            If BinaryArray = 1 Then
                Binary1(i) = tmpByte
            Else
                Binary2(i) = tmpByte
            End If

        Next i

        r.Close()
        fs.Close()

    End Sub

    Public Function Get_OSID_Compare(ByVal File_Name As String, Optional ByVal Return_Position As Boolean = False) As String

        Dim fs As New IO.FileStream(File_Name, IO.FileMode.Open, IO.FileAccess.Read)
        Dim r As New IO.BinaryReader(fs)
        Dim mFile_Info As New IO.FileInfo(File_Name)

        Dim OSID As String = ""

        Dim OSID_String1 As String = ""
        Dim OSID_Start1 As String = ""

        Dim OSID_String2 As String = ""
        Dim OSID_Start2 As String = ""

        Dim Use_OSID As String = ""
        Dim tmpBinaryValue As String = ""

        Dim CellValue As String = ""
        Dim i As Integer = 0
        Dim c1 As Integer
        Dim c2 As Integer

        Try
            For i = 0 To mFile_Info.Length - 1

                tmpBinaryValue = CellValue
                CellValue = Hex(r.ReadByte)  'BinaryFile(i))

                If CellValue.Length < 2 Then
                    CellValue = "0" & CellValue
                End If

                tmpBinaryValue += CellValue

                If i > (mFile_Info.Length - 6) And c1 < 3 Then
                    OSID_String1 += CellValue
                    If c1 = 0 Then OSID_Start1 = (mFile_Info.Length - 6).ToString
                    c1 += 1
                End If
                If i > (mFile_Info.Length - 8) And c2 < 3 Then
                    OSID_String2 += CellValue
                    If c2 = 0 Then OSID_Start2 = (mFile_Info.Length - 8).ToString
                    c2 += 1
                End If

            Next

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error Retrieving OSID", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
        fs.Close()
        r.Close()

        Try
            If tmpBinaryValue = "FFFF" Or tmpBinaryValue = "0000" Then
                If Not Return_Position Then
                    Return Convert.ToInt32(OSID_String2, 16)
                Else
                    Return OSID_Start2
                End If
            Else
                If Not Return_Position Then
                    Return Convert.ToInt32(OSID_String1, 16)
                Else
                    Return OSID_Start1
                End If
            End If
        Catch ex As Exception
            Return ""
        End Try

    End Function
End Module
